
import javax.swing.JPanel;
import java.awt.geom.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.JOptionPane;
import javax.swing.JMenuItem;

/**
 * Clase de dibujo del pilar.
 *
 * @author Agustin Gonzales, Victor Arredondo, Yen-kung Yu
 */
class GPilar extends Pilar implements Drawable {

    private static final double xPoints[] = {0, 0.10, 0.125, 0.175, 0.225, 0.275, 0.325,
        0.375, 0.425, 0.475, 0.525, 0.575, 0.625, 0.675, 0.725, 0.775, 0.825, 0.875, 0.90, 1.0};
    private static final double yPoints[] = {0, 0.10,0.10,0.10,0.10,0.10,0.10,0.10,
		0.10,0.10,0.10,0.10,0.10,0.10,0.10,0.10,0.10,0.10,0.10,0.10};
		
    private static final Path2D.Double polyline =
            new Path2D.Double(Path2D.WIND_EVEN_ODD, xPoints.length);
    private Path2D.Double shape;
    private Stroke stroke;
    private Vector2D freeAendPosition;    // to draw a spring we need
    private Vector2D freeBendPosition;  // extremes well definied
    private MyWorld wi;
    private PhysicsLabPanel panel;

    static {  // static initialization block. It creates a spring of length = 1.
        polyline.moveTo(xPoints[0], yPoints[0]);
        for (int index = 1; index < xPoints.length; index++) {
            polyline.lineTo(xPoints[index], yPoints[index]);
        }
    }

    /**
     * Constructor GPilar. Inicializa un nuevo GPilar.
     *
     * @param largonat Largo natural del Pilar.
     * @param rigidez Constante de fuerza del Pilar.
     * @param posA Posicion vectorial del extremo A del Pilar.
     * @param posB Posicion vectorial del extremo B del Pilar.
     * @param w Llama al metodo stop() para detener el sistema.
     * @param lab sin uso.
     */
    public GPilar(float largonat, float rigidez, Vector2D posA, Vector2D posB, MyWorld w, PhysicsLabPanel lab) {
        super(largonat, rigidez);
        wi = w;
        panel = lab;
        freeAendPosition = new Vector2D((double) posA.getX(), (double) posA.getY());
        freeBendPosition = new Vector2D((double) posB.getX(), (double) posB.getY());
        AffineTransform at = AffineTransform.getTranslateInstance(0, 0);
        Vector2D v = getVector();
        at.rotate(v.getX(), v.getY());
        at.scale(v.module(), largonat);
        shape = (Path2D.Double) at.createTransformedShape(polyline);
        stroke = new BasicStroke(15);
    }

    /**
     * Obtiene la posicion del extremo A del Pilar.
     *
     * @return Posicion vectorial.
     */
    public Vector2D getAendPosition() {
        if (a_end == null) {
            return freeAendPosition;
        }
        return super.getAendPosition();
    }

    /**
     * Obtiene la posicion del extremo B del Pilar.
     *
     * @return Posicion vectorial.
     */
    public Vector2D getBendPosition() {
        if (b_end == null) {
            return freeBendPosition;
        }
        return super.getBendPosition();
    }

    /**
     * Dibuja el Pilar.
     *
     * @param g Objeto grafico de 2 dimensiones.
     */
    public void draw(Graphics2D g) {
        Vector2D a = getAendPosition();
        double ax = a.getX();
        double ay = a.getY();
        Vector2D v = getVector();
        AffineTransform at = AffineTransform.getTranslateInstance(ax, ay);
        at.rotate(v.getX(), v.getY());
        at.scale(v.module(), largonat);
        shape = (Path2D.Double) at.createTransformedShape(polyline);
        if (v.module() < 5 * largonat) {
            g.setColor(Color.GRAY);
        } else {
            wi.stop();
            //panel.message();
            g.setColor(Color.RED);
           // detachBend();
           // detachAend();
        }
        g.setStroke(stroke);
        g.draw(shape);
    }

    /**
     * Determina si se contiene un objeto.
     *
     * @param x Coordenada X de la posicion.
     * @param y Coordenada Y de la posicion.
     * @return True si contiene a otro, False si no.
     */
    public boolean contains(int x, int y) {
        return shape.getBounds2D().contains(x, y);
    }

    /**
     * Establece la posicion siguiente del bloque.
     *
     * @param x Coordenada X de la posicion.
     * @param y Coordenada Y de la posicion.
     */
    public void dragTo(int x, int y) {

        Vector2D a = getAendPosition();
        Vector2D b = getBendPosition();
        double da = Math.abs(a.getX() - x) + Math.abs(a.getY() - y); // "distance" to a
        double db = Math.abs(b.getX() - x) + Math.abs(b.getY() - y); // "distance" to a
        if (da < db) {
            detachAend();
            freeAendPosition.set(x, y);
        } else {
            detachBend();
            freeBendPosition.set(x, y);
        }
    }

    /**
     * Actualiza el estado del los extremos.
     *
     */
    public void updateState() {  // just to return to a rest length
        if (a_end == null && b_end == null) {
            return;
        } // when it is released
        if (a_end != null) {
            Vector2D v = getVector().unitary().times(largonat);
            freeBendPosition = getAendPosition().plus(v);
            return;
        }
        if (b_end != null) {
            Vector2D v = getVector().unitary().times(largonat);
            freeAendPosition = getBendPosition().minus(v);
            return;
        }
    }

    /**
     * Indica que el bloque se ha seleccionado.
     */
    public void setSelected() {
        stroke = new BasicStroke(15);
    }

    /**
     * Se deja de marcar el bloque como seleccionado.
     */
    public void setReleased() {
        stroke = new BasicStroke(12);
    }
}
